@@ -98,7 +98,7 @@ typedef struct param_t {
     int             iMaxXferMBperSec;
     int             fRW_DataCombined;
     uint32_t        iPeriod_ms;
-    GdkColor        aoColor[NMONITORS];
+    GdkRGBA         aoColor[NMONITORS];
 } param_t;
 
 typedef struct color_selector_t {
@@ -326,15 +326,32 @@ static int SetSingleBarColor (struct diskperf_t *p_poPlugin, int p_iBar)
     Widget_t       *pwBar;
 
     pwBar = poMonitor->aoPerfBar[p_iBar].pwBar;
-	gtk_widget_modify_bg(GTK_WIDGET(*pwBar),
+#if GTK_CHECK_VERSION (3, 16, 0)
+    GtkCssProvider *css_provider;
+#if GTK_CHECK_VERSION (3, 20, 0)
+    gchar * css = g_strdup_printf("progressbar progress { background-color: %s; background-image: none; }", gdk_rgba_to_string(&poConf->aoColor[p_iBar]));
+#else
+    gchar * css = g_strdup_printf(".progressbar { background-color: %s; background-image: none; }", gdk_rgba_to_string(&poConf->aoColor[p_iBar]));
+#endif
+    /* Setup Gtk style */
+    css_provider = gtk_css_provider_new ();
+    gtk_css_provider_load_from_data (css_provider, css, strlen(css), NULL);
+    gtk_style_context_add_provider (
+        GTK_STYLE_CONTEXT (gtk_widget_get_style_context (GTK_WIDGET (*pwBar))),
+        GTK_STYLE_PROVIDER (css_provider),
+        GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+    g_free(css);
+#else
+	gtk_widget_override_background_color(GTK_WIDGET(*pwBar),
 						 GTK_STATE_PRELIGHT,
 						 &poConf->aoColor[p_iBar]);
-	gtk_widget_modify_bg(GTK_WIDGET(*pwBar),
+	gtk_widget_override_background_color(GTK_WIDGET(*pwBar),
 						 GTK_STATE_SELECTED,
 						 &poConf->aoColor[p_iBar]);
-	gtk_widget_modify_base(GTK_WIDGET(*pwBar),
+	gtk_widget_override_color(GTK_WIDGET(*pwBar),
 						 GTK_STATE_SELECTED,
 						 &poConf->aoColor[p_iBar]);
+#endif
     return (0);
 }				/* SetSingleBarColor() */
 
@@ -387,7 +404,7 @@ static int CreateMonitorBars (struct diskperf_t *p_poPlugin,
     Widget_t       *pwBar;
     int             i;
 
-    poMonitor->wBox = xfce_hvbox_new (p_iOrientation, FALSE, 0);
+    poMonitor->wBox = gtk_box_new (p_iOrientation, 0);
     gtk_widget_show (poMonitor->wBox);
 
     gtk_container_add (GTK_CONTAINER (poMonitor->wEventBox),
@@ -402,11 +419,11 @@ static int CreateMonitorBars (struct diskperf_t *p_poPlugin,
     for (i = 0; i < 2; i++) {
 	pwBar = poMonitor->awProgressBar + i;
 	*pwBar = GTK_WIDGET (gtk_progress_bar_new ());
-	gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR (*pwBar),
-					  (p_iOrientation ==
-					   GTK_ORIENTATION_HORIZONTAL ?
-					   GTK_PROGRESS_BOTTOM_TO_TOP :
-					   GTK_PROGRESS_LEFT_TO_RIGHT));
+	gtk_orientable_set_orientation (GTK_ORIENTABLE(*pwBar), !p_iOrientation);
+	gtk_progress_bar_set_inverted (GTK_PROGRESS_BAR(*pwBar),
+					   (p_iOrientation ==
+					    GTK_ORIENTATION_HORIZONTAL));
+
 	if ((i == 1) && poConf->fRW_DataCombined)
 	    gtk_widget_hide (GTK_WIDGET (*pwBar));
 	else
@@ -459,9 +476,9 @@ static diskperf_t *diskperf_create_control (XfcePanelPlugin *plugin)
 
     poConf->fTitleDisplayed = 1;
 
-    gdk_color_parse ("#0000FF", poConf->aoColor + R_DATA);
-    gdk_color_parse ("#FF0000", poConf->aoColor + W_DATA);
-    gdk_color_parse ("#00FF00", poConf->aoColor + RW_DATA);
+    gdk_rgba_parse (poConf->aoColor + R_DATA, "#0000FF");
+    gdk_rgba_parse (poConf->aoColor + W_DATA, "#FF0000");
+    gdk_rgba_parse (poConf->aoColor + RW_DATA, "#00FF00");
 
     poConf->iMaxXferMBperSec = 40;
     poConf->fRW_DataCombined = 1;
@@ -586,15 +603,15 @@ static void diskperf_read_config (XfcePanelPlugin *plugin,
         xfce_rc_read_int_entry (rc, (CONF_MONITOR_BAR_ORDER), RW_ORDER);
 
     if ((value = xfce_rc_read_entry (rc, (CONF_READ_COLOR), NULL))) {
-        gdk_color_parse (value, poConf->aoColor + R_DATA);
+        gdk_rgba_parse (poConf->aoColor + R_DATA, value);
     }
 
     if ((value = xfce_rc_read_entry (rc, (CONF_WRITE_COLOR), NULL))) {
-        gdk_color_parse (value, poConf->aoColor + W_DATA);
+        gdk_rgba_parse (poConf->aoColor + W_DATA, value);
     }
 
     if ((value = xfce_rc_read_entry (rc, (CONF_READ_WRITE_COLOR), NULL))) {
-        gdk_color_parse (value, poConf->aoColor + RW_DATA);
+        gdk_rgba_parse (poConf->aoColor + RW_DATA, value);
     }
     SetMonitorBarColor (poPlugin);
 
@@ -609,9 +626,6 @@ static void diskperf_write_config (XfcePanelPlugin *plugin,
 	/* Write diskperf configuration into xml file */
 {
     struct param_t *poConf = &(poPlugin->oConf.oParam);
-    const char     *acColorFormat = "#%02X%02X%02X";
-    GdkColor       *poColor;
-    char            acBuffer[16];
     XfceRc *rc;
     char *file;
 
@@ -642,20 +656,9 @@ static void diskperf_write_config (XfcePanelPlugin *plugin,
     xfce_rc_write_int_entry (rc, CONF_MONITOR_BAR_ORDER, 
                              poConf->eMonitorBarOrder);
 
-    poColor = poConf->aoColor + R_DATA;
-    snprintf (acBuffer, 16, acColorFormat,
-	     poColor->red >> 8, poColor->green >> 8, poColor->blue >> 8);
-    xfce_rc_write_entry (rc, CONF_READ_COLOR, acBuffer);
-
-    poColor = poConf->aoColor + W_DATA;
-    snprintf (acBuffer, 16, acColorFormat,
-	     poColor->red >> 8, poColor->green >> 8, poColor->blue >> 8);
-    xfce_rc_write_entry (rc, CONF_WRITE_COLOR, acBuffer);
-
-    poColor = poConf->aoColor + RW_DATA;
-    snprintf (acBuffer, 16, acColorFormat,
-	     poColor->red >> 8, poColor->green >> 8, poColor->blue >> 8);
-    xfce_rc_write_entry (rc, CONF_READ_WRITE_COLOR, acBuffer);
+    xfce_rc_write_entry (rc, CONF_READ_COLOR, gdk_rgba_to_string(poConf->aoColor + R_DATA));
+    xfce_rc_write_entry (rc, CONF_WRITE_COLOR, gdk_rgba_to_string(poConf->aoColor + W_DATA));
+    xfce_rc_write_entry (rc, CONF_READ_WRITE_COLOR, gdk_rgba_to_string(poConf->aoColor + RW_DATA));
 
     xfce_rc_close (rc);
 }				/* diskperf_write_config() */
@@ -834,11 +837,8 @@ static void ChooseColor (Widget_t p_wPB, void *p_pvPlugin)
     struct diskperf_t *poPlugin = (diskperf_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct gui_t   *poGUI = &(poPlugin->oConf.oGUI);
-    Widget_t        wDialog;
-    GdkColor       *poColor;
-    GtkColorSelection *colorsel;
+    GdkRGBA         poColor;
     int             iPerfBar;
-    int             iResponse;
 
     if (p_wPB == poGUI->wPB_Rcolor)
 	iPerfBar = R_DATA;
@@ -848,26 +848,10 @@ static void ChooseColor (Widget_t p_wPB, void *p_pvPlugin)
 	iPerfBar = RW_DATA;
     else
 	return;
-    poColor = poConf->aoColor + iPerfBar;
-
-    wDialog = gtk_color_selection_dialog_new (_("Select color"));
-    gtk_window_set_transient_for (GTK_WINDOW (wDialog),
-				  GTK_WINDOW (poPlugin->oConf.wTopLevel));
-    colorsel = GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (wDialog)->
-				    colorsel);
-    gtk_color_selection_set_previous_color (colorsel, poColor);
-    gtk_color_selection_set_current_color (colorsel, poColor);
-    gtk_color_selection_set_has_palette (colorsel, TRUE);
-
-    iResponse = gtk_dialog_run (GTK_DIALOG (wDialog));
-    if (iResponse == GTK_RESPONSE_OK) {
-	gtk_color_selection_get_current_color (colorsel, poColor);
-	gtk_widget_modify_bg (poPlugin->oConf.aoColorWidgets[iPerfBar].wDA,
-			      GTK_STATE_NORMAL, poColor);
-	SetMonitorBarColor (poPlugin);
-    }
-
-    gtk_widget_destroy (wDialog);
+    gtk_color_chooser_get_rgba(GTK_COLOR_CHOOSER(p_wPB), &poColor);
+    DBG("color changed to %s for monitor %d", gdk_rgba_to_string(&poColor), iPerfBar);
+    poConf->aoColor[iPerfBar] = poColor;
+    SetMonitorBarColor (poPlugin);
 }				/* ChooseColor() */
 
 	/**************************************************************/
@@ -969,7 +953,6 @@ static void diskperf_create_options (XfcePanelPlugin *plugin,
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct gui_t   *poGUI = &(poPlugin->oConf.oGUI);
     char            acBuffer[16];
-    struct color_selector_t *poColorWidgets;
     Widget_t       *apwColorPB[NMONITORS];
     int             i;
 
@@ -981,8 +964,8 @@ static void diskperf_create_options (XfcePanelPlugin *plugin,
     
     dlg = xfce_titled_dialog_new_with_buttons (_("Disk Performance Monitor"),
                                                 GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
-                                                GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
-                                                GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
+                                                GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                "gtk-close", GTK_RESPONSE_OK,
                                                 NULL);
     
     g_signal_connect (G_OBJECT (dlg), "response",
@@ -991,10 +974,10 @@ static void diskperf_create_options (XfcePanelPlugin *plugin,
     gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER);
     gtk_window_set_icon_name (GTK_WINDOW (dlg), "drive-harddisk");
     
-    vbox = gtk_vbox_new(FALSE, BORDER);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, BORDER);
     gtk_container_set_border_width (GTK_CONTAINER (vbox), BORDER - 2);
     gtk_widget_show(vbox);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), vbox,
+    gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG (dlg))), vbox,
                         TRUE, TRUE, 0);
     
     poPlugin->oConf.wTopLevel = dlg;
@@ -1070,14 +1053,8 @@ static void diskperf_create_options (XfcePanelPlugin *plugin,
     apwColorPB[W_DATA] = &(poGUI->wPB_Wcolor);
     apwColorPB[RW_DATA] = &(poGUI->wPB_RWcolor);
     for (i = 0; i < NMONITORS; i++) {
-	poColorWidgets = poPlugin->oConf.aoColorWidgets + i;
-	poColorWidgets->wDA = gtk_drawing_area_new ();
-	gtk_widget_modify_bg (poColorWidgets->wDA, GTK_STATE_NORMAL,
-			      poConf->aoColor + i);
-	gtk_container_add (GTK_CONTAINER (*(apwColorPB[i])),
-			   poColorWidgets->wDA);
-	gtk_widget_show (GTK_WIDGET (poColorWidgets->wDA));
-	g_signal_connect (GTK_WIDGET (*(apwColorPB[i])), "clicked",
+	gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(*(apwColorPB[i])), poConf->aoColor + i);
+	g_signal_connect (GTK_WIDGET (*(apwColorPB[i])), "color-set",
 			  G_CALLBACK (ChooseColor), poPlugin);
     }
 		      
